//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "ClientList.h"
#include "emule.h"
#include "otherfunctions.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CClientList::CClientList() : m_clientList(100)
{
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CClientList::~CClientList()
{
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CClientList::GetStatistics(uint32 &totalclient, uint32 stats[], uint32 clientsPorts[], ClientsData *pData, uint32 &totalMODs, CMap<POSITION, POSITION, uint32, uint32> *MODs, CMap<int, int, uint32, uint32> *pCountries)
{
	EMULE_TRY
	if (!pData)
		return;

	POSITION			pos_MOD;
	CUpDownClient		*pClient;
	CString				strMODName;
	uint32				dwCount;
	int					iCountryIdx;

	totalclient = m_clientList.GetCount();

	for (int i = 0;i < SO_LAST;i++)
	{
		pData->m_pClients[i].RemoveAll();
		stats[i] = 0;
	}
	pData->m_pClients[i].RemoveAll(); // reset the last one

//	Reset ports information
	clientsPorts[0] = 0;
	clientsPorts[1] = 0;

//	Reset MODs variables
	totalMODs = 0;
	MODs->RemoveAll();

//	Reset Countries variables
	pCountries->RemoveAll();

//	Reset SUI, LowID and Problematic clients info
	stats[14] = 0;	//	SUI success
	stats[15] = 0;	//	SUI failed
	stats[16] = 0;	//	LowID
	stats[17] = 0;	//	Problematic clients (DS_ERROR || US_ERROR)

	for (POSITION pos = m_clientList.GetHeadPosition(); pos != NULL; )
	{
		pClient = m_clientList.GetNext(pos);

		int		iVersion = pClient->GetVersion();

		switch (pClient->GetClientSoft())
		{
			case SO_UNKNOWN :
				stats[SO_UNKNOWN]++;
				break;
			case SO_EDONKEY :
				stats[SO_EDONKEY]++;
				(pData->m_pClients[SO_EDONKEY])[iVersion]++;
				break;
			case SO_EDONKEYHYBRID :
				stats[SO_EDONKEYHYBRID]++;
				(pData->m_pClients[SO_EDONKEYHYBRID])[iVersion]++;
				break;
			case SO_EMULE :
				stats[SO_EMULE]++;
				(pData->m_pClients[SO_EMULE])[iVersion]++;
				break;
			case SO_PLUS :
				stats[SO_PLUS]++;
				(pData->m_pClients[SO_PLUS])[iVersion]++;
				break;
			case SO_OLDEMULE:
				stats[SO_OLDEMULE]++;
				(pData->m_pClients[SO_OLDEMULE])[iVersion]++;
				break;
			case SO_AMULE:
				stats[SO_AMULE]++;
				(pData->m_pClients[SO_AMULE])[iVersion]++;
				break;
			case SO_SHAREAZA:
				stats[SO_SHAREAZA]++;
				(pData->m_pClients[SO_SHAREAZA])[iVersion]++;
				break;
			case SO_XMULE:
				stats[SO_XMULE]++;
				(pData->m_pClients[SO_XMULE])[iVersion]++;
				break;
			case SO_MLDONKEY:
				stats[SO_MLDONKEY]++;
				(pData->m_pClients[SO_MLDONKEY])[iVersion]++;
				break;
			case SO_LPHANT:
				stats[SO_LPHANT]++;
				(pData->m_pClients[SO_LPHANT])[iVersion]++;
				break;
		}

		(pData->m_pClients[SO_LAST])[pClient->GetDownloadState()]++;

		if (pClient->Credits() != NULL)
		{
			switch(pClient->Credits()->GetCurrentIdentState(pClient->GetIP()))
			{
				case IS_IDENTIFIED:
					stats[14]++;
					break;
				case IS_IDFAILED:
				case IS_IDNEEDED:
				case IS_IDBADGUY:
					stats[15]++;
			}
		}

		if (pClient->HasLowID())
		{
			stats[16]++;
		}

		if (pClient->GetDownloadState() == DS_ERROR || pClient->GetUploadState() == US_ERROR )
			stats[17]++;

		switch (pClient->GetUserPort())
		{
		//	Default Port
			case 4662:
				++clientsPorts[0];
				break;
		//	Other Port
 	 		default:
				++clientsPorts[1];
		}

	//	Count MODs
		if ((pClient->GetClientSoft() == SO_EMULE) || (pClient->GetClientSoft() == SO_OLDEMULE))
		{
			strMODName = pClient->GetModString();
			if (!strMODName.IsEmpty())
			{
				totalMODs++;
				pos_MOD = liMODsTypes.Find(strMODName);
				if (!pos_MOD)
				{
					pos_MOD = liMODsTypes.AddTail(strMODName);
					MODs->SetAt(pos_MOD, 1);
				}
				else
				{
					dwCount = 0;
					MODs->Lookup(pos_MOD, dwCount);
					MODs->SetAt(pos_MOD, ++dwCount);
				}
			}
		}

	//	Count Countries
		CMap<int, int, uint32, uint32>::CPair *pPair;

		iCountryIdx = pClient->GetCountryIndex();
		pPair = pCountries->PLookup(iCountryIdx);
		if (pPair != NULL)
			pPair->value++;
		else
			pCountries->SetAt(iCountryIdx, 1);
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CClientList::AddClient(CUpDownClient* toadd, bool bSkipDupTest)
{
	EMULE_TRY

	if (toadd == NULL)
		return false;

	if ( !bSkipDupTest)
	{
		if (m_clientList.Find(toadd))
			return false;
	}
	if (m_pctlClientList != NULL && m_pctlClientList->m_hWnd != NULL)
		m_pctlClientList->AddClient(toadd);
	m_clientList.AddTail(toadd);

	return true;

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CClientList::RemoveClient(CUpDownClient *pClient)
{
	EMULE_TRY

	POSITION		pos = m_clientList.Find(pClient);

	if (pos != NULL)
	{
	//	Just to be sure...
		g_eMuleApp.m_pUploadQueue->RemoveFromUploadQueue(pClient, ETS_DISCONNECT);
		g_eMuleApp.m_pUploadQueue->RemoveFromWaitingQueue(pClient);
		g_eMuleApp.m_pDownloadQueue->RemoveSource(pClient);
		m_pctlClientList->RemoveClient(pClient);
		m_clientList.RemoveAt(pos);
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CClientList::DeleteAll()
{
	EMULE_TRY

	g_eMuleApp.m_pUploadQueue->DeleteAll();
	g_eMuleApp.m_pDownloadQueue->DeleteAll();

	POSITION			pos1, pos2;
	CUpDownClient	   *pClient;

	for (pos1 = m_clientList.GetHeadPosition(); (pos2 = pos1) != NULL; )
	{
		pClient = m_clientList.GetNext(pos1);
		m_clientList.RemoveAt(pos2);
		if (pClient)
			delete pClient;
	}
	liMODsTypes.RemoveAll();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef OLD_SOCKETS_ENABLED
bool CClientList::AttachToAlreadyKnown(CUpDownClient **ppClient, CClientReqSocket *pSocket)
{
	EMULE_TRY

	CUpDownClient	   *pClient = (*ppClient);
	CUpDownClient	   *pClient2;

	for (POSITION pos = m_clientList.GetHeadPosition(); pos != NULL; )
	{

		pClient2 = m_clientList.GetNext(pos);

	//	I dont know how and why this can happen... but eMule crashed here all the time for me!
		if (pClient2 != NULL && pClient != pClient2 && pClient->Compare(pClient2) != 0)
		{
			ASSERT ( (pClient != pClient2) );
			if (pSocket != NULL)
			{
				if (pClient2->m_pRequestSocket != NULL)
				{
					pClient2->m_pRequestSocket->Safe_Delete();
				}
				pClient2->m_pRequestSocket = pSocket;
				pClient->m_pRequestSocket = NULL;
			}
			*ppClient = NULL;
			delete pClient;
			*ppClient = pClient2;
			return true;
		}
	}

	EMULE_CATCH

	return false;
}
#endif //OLD_SOCKETS_ENABLED
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CUpDownClient* CClientList::FindClientByIP(uint32 clientip, uint16 port)
{
	EMULE_TRY

	CUpDownClient	   *pClient;

	for (POSITION pos = m_clientList.GetHeadPosition(); pos != NULL; )
	{
		pClient = m_clientList.GetNext(pos);
		if (pClient)
			if (pClient->GetIP() == clientip && pClient->GetUserPort() == port)
				return pClient;
	}

	EMULE_CATCH

	return NULL;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CUpDownClient* CClientList::FindClientByUserHash(uchar* clienthash)
{
	EMULE_TRY

	CUpDownClient	   *pClient;

	for (POSITION pos = m_clientList.GetHeadPosition(); pos != NULL; )
	{
		pClient = m_clientList.GetNext(pos);
		if (pClient)
			if (!md4cmp(pClient->GetUserHash() , clienthash))
				return pClient;
	}

	EMULE_CATCH

	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CUpDownClient* CClientList::FindClientByPortAndID(uint32 dwUserID,uint16 nPort)
{
	EMULE_TRY

	CUpDownClient	   *pClient;

	for (POSITION pos = m_clientList.GetHeadPosition(); pos != NULL; )
	{
		pClient = m_clientList.GetNext(pos);
		if(pClient)
			if (pClient->GetUserID() == dwUserID && pClient->GetUserPort() == nPort)
				return pClient;
	}

	EMULE_CATCH

	return NULL;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef OLD_SOCKETS_ENABLED
void CClientList::Debug_SocketDeleted(CClientReqSocket *pSocket)
{
	EMULE_TRY

	CUpDownClient	   *pClient;

	for (POSITION pos = m_clientList.GetHeadPosition(); pos != NULL; )
	{

		pClient = m_clientList.GetNext(pos);

		if (!AfxIsValidAddress(pClient, sizeof(CUpDownClient)))
		{
			AfxDebugBreak();
		}
		if (pClient && pClient->m_pRequestSocket == pSocket)
		{
		//	AfxDebugBreak();
		}
	}

	EMULE_CATCH
}
#endif //OLD_SOCKETS_ENABLED
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CClientList::IsValidClient(CUpDownClient *pClient)
{
	EMULE_TRY

	if (pClient == NULL)
		return false;

	return m_clientList.Find(pClient);

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CClientList::GetClientListByFileID(CTypedPtrList<CPtrList, CUpDownClient*> *m_pClientList, const uchar *fileid)
{
	EMULE_TRY

	m_pClientList->RemoveAll();

	CUpDownClient	   *pClient;

	for (POSITION pos = m_clientList.GetHeadPosition(); pos != NULL; )
	{
		pClient = m_clientList.GetNext(pos);
		if (md4cmp(pClient->GetUploadFileID(), fileid) == 0)
			m_pClientList->AddTail(pClient);
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint32 CClientList::GetA4AFSourcesCount()
{
	EMULE_TRY

	CUpDownClient	   *pClient;
	uint32				dwCountA4AF = 0;

	for (POSITION pos = m_clientList.GetHeadPosition(); pos != NULL; )
	{
		pClient = m_clientList.GetNext(pos);
		if (!pClient->m_otherRequestsList.IsEmpty())
			dwCountA4AF++;
	}

	return dwCountA4AF;

	EMULE_CATCH
	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CClientList::UpdateClient(CUpDownClient *pClient)
{
	if (pClient != NULL)
	{
		POSITION		posClient = m_clientList.Find(pClient);

	//	If the client is in the list...
		if (posClient != NULL)
		{
			g_eMuleApp.m_pdlgEmule->m_wndTransfer.m_ctlClientList.UpdateClient(pClient);
			g_eMuleApp.m_pDownloadList->UpdateSource(pClient);
		}
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CClientList::ResetIP2Country()
{
	CUpDownClient *pClient;

	for(POSITION pos = m_clientList.GetHeadPosition(); pos != NULL; m_clientList.GetNext(pos))
	{ 
		pClient = g_eMuleApp.m_pClientList->m_clientList.GetAt(pos); 
		pClient->ResetIP2Country();
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
