//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//	ListenSocket.cpp : implementation file
//

#include "stdafx.h"
#include "emule.h"
#include "ListenSocket.h"
#include "opcodes.h"
#include "KnownFile.h"
#include "SharedFileList.h"
#include "UploadQueue.h"
#include "updownclient.h"
#include "ClientList.h"
#include "otherfunctions.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif

#ifdef OLD_SOCKETS_ENABLED

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	CClientReqSocket
CClientReqSocket::CClientReqSocket(CPreferences* in_prefs, CUpDownClient* in_client)
{
	m_pPrefs = in_prefs;
	m_pClient = in_client;
	if (in_client)
		m_pClient->m_pRequestSocket = this;
	g_eMuleApp.m_pListenSocket->AddSocket(this);
	ResetTimeOutTimer();
	m_bDeleteThis = false;
	m_dwDeleteTimer = 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CClientReqSocket::~CClientReqSocket()
{
	if (m_pClient != NULL)
		m_pClient->m_pRequestSocket = NULL;
	m_pClient = NULL;
	g_eMuleApp.m_pListenSocket->RemoveSocket(this);

	DEBUG_ONLY (g_eMuleApp.m_pClientList->Debug_SocketDeleted(this));
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CClientReqSocket::ResetTimeOutTimer()
{
	m_dwTimeoutTimer = ::GetTickCount();
};
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	CheckTimeOut() checks the timeout timer, and, if it has expired, it disconnects the m_pRequestSocket
//	and resets the timeout timer. It returns true if the timer has expired, false if it has not.
bool CClientReqSocket::CheckTimeOut()
{
//	increase socket timeout for DL clients
	 if (m_pClient != NULL && m_pClient->GetDownloadState() == DS_DOWNLOADING)
	 	return false;
//	If the timeout period has been exceeded...
	if (::GetTickCount() - m_dwTimeoutTimer > CONNECTION_TIMEOUT)
	{
	//	Reset the timeout timer
		m_dwTimeoutTimer = ::GetTickCount();
		Disconnect();
		return true;
	}
	return false;
};
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CClientReqSocket::OnClose(int nErrorCode)
{
	ASSERT (g_eMuleApp.m_pListenSocket->IsValidSocket(this));
	CEMSocket::OnClose(nErrorCode);
	Disconnect();
};
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CClientReqSocket::Disconnect()
{
//	Tell the m_pRequestSocket not to receive any more notifications
	AsyncSelect(0);
	m_eConnectionState = ES_DISCONNECTED;
	if (m_pClient != NULL && !g_eMuleApp.m_pClientList->IsValidClient(m_pClient))
	{
		AddDebugLogLine(_T("Invalid client '%s'"), m_pClient->GetUserName());
		m_pClient = NULL;
	}
	if (m_pClient == NULL)
	{
		Safe_Delete();
	}
	else
	{
		if (m_pClient->Disconnected())
			m_pClient = NULL;
	}
};
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CClientReqSocket::TimedDelete()
{
//	It seems that MFC Sockets call socketfunctions after they are deleted, even if the m_pRequestSocket is closed
//	and select(0) is set. So we need to wait some time to make sure this doesn't happen.
	if (::GetTickCount() - m_dwDeleteTimer > 10000)
		delete this;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CClientReqSocket::Safe_Delete()
{
	ASSERT (g_eMuleApp.m_pListenSocket->IsValidSocket(this));
	AsyncSelect(0);
	m_dwDeleteTimer = ::GetTickCount();
	if (m_SocketData.hSocket != INVALID_SOCKET)
		ShutDown(2);
	if (m_pClient != NULL)
		m_pClient->m_pRequestSocket = NULL;
	m_pClient = NULL;
	m_eConnectionState = ES_DISCONNECTED;
	m_bDeleteThis = true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CClientReqSocket::ProcessPacket(char *pcPacketBuf, uint32 dwPacketSize, EnumOpcodes eOpcode)
{
	EnumDLQState enumClientDLState;

	try
	{
		try
		{
			if (m_pClient == NULL && eOpcode != OP_HELLO)
			{
				throw CString(_T("a client asks for something without saying hello"));
			}

			switch (eOpcode)
			{
				case OP_HELLOANSWER:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(dwPacketSize);
					m_pClient->ProcessHelloAnswer(pcPacketBuf, dwPacketSize);
					if (m_pClient)
					{
						m_pClient->ConnectionEstablished();
					//	Update of client in Clientlist will lead to GUI update in ClientListCtrl as well as DownloadListCtrl
						g_eMuleApp.m_pClientList->UpdateClient(m_pClient);
					}
					break;
				}
				case OP_HELLO:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(dwPacketSize);
					bool newClient = false;
					if (!m_pClient)
					{
					//	Create new client to save standard informations
						m_pClient = new CUpDownClient(this);
						newClient = true;
					}
				//	Originally we had try & catch here, but because our EMULE_TRY & EMULE_CATCH
				//	function in ProcessHelloPacket, local try & catch did work. That means even in case of exception
				//	we process the client without full information.
					if (!m_pClient->ProcessHelloPacket(reinterpret_cast<BYTE*>(pcPacketBuf), dwPacketSize))
					{
						if (newClient)
							safe_delete(m_pClient);
						throw CString(_T("unable to process HelloPacket"));
					}
				//	Now we check if we know this client already. If yes this m_pRequestSocket will be attached
				//	to the known client, the new client will be deleted and the var. "client" will point
				//	to the known client. If not we keep our new-constructed client.
					if (g_eMuleApp.m_pClientList->AttachToAlreadyKnown(&m_pClient, this))
					{
					//	Update the old client informations
						m_pClient->ProcessHelloPacket(reinterpret_cast<BYTE*>(pcPacketBuf), dwPacketSize);
						m_pClient->DisableL2HAC();
					}
					else
					{
					//	If new client was not created delete him
						if (!g_eMuleApp.m_pClientList->AddClient(m_pClient))
						{
							safe_delete(m_pClient);
							throw CString(_T("unable to add client in ClientList"));
						}
						m_pClient->SetCommentDirty();
					}

				//	Update of client in Clientlist will lead to GUI update in ClientListCtrl as well as DownloadListCtrl
					g_eMuleApp.m_pClientList->UpdateClient(m_pClient);

				//	Send a response packet with standard information
					if (m_pClient->GetHashType() == SO_EMULE)
						m_pClient->SendMuleInfoPacket(false);

					m_pClient->SendHelloAnswer();
					if (m_pClient)
						m_pClient->ConnectionEstablished();

					break;
				}
				case OP_REQUESTFILENAME:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadFileRequest(dwPacketSize);

				//	IP banned, no answer for this request
					if (m_pClient->IsBanned())
					{
						break;
					}

				//	If we're filtering scanning (and misbehaving) clients and this client has asked for a
				//	non-existant file more than three times...
					if (g_eMuleApp.m_pGlobPrefs->IsScanFilterEnabled() && m_pClient->GetFailedFileRequests() >= 3)
					{
					//	Temporarily ban the client from the upload queue
						g_eMuleApp.m_pIPFilter->AddTemporaryBannedIP(m_pClient->GetIP());
					//	If we're logging countermeasures...
						if (!g_eMuleApp.m_pGlobPrefs->IsCMNotLog())
							AddDebugLogLine( RGB_LOG_DIMMED + _T("Client '%s' (%s) (IP: %s) added to filtered clients due to file scanning"),
											 m_pClient->GetUserName(), m_pClient->GetClientNameAndVersionString(), m_pClient->GetFullIP() );
						Disconnect();
						break;
					}

				//	Since we don't process extended information it is better for compatibility with
				//	the rest of the network to always process a filehash & extended information
				//	only if the file exists and was activated by tag ET_EXTENDEDREQUEST
					if (dwPacketSize >= 16)
					{

						uchar		requestedFileHash[16];

						md4cpy(requestedFileHash, pcPacketBuf);

						CKnownFile	*m_pReqSharedFile = g_eMuleApp.m_pSharedFilesList->GetFileByID(requestedFileHash);

					//	If we are not sharing the requested file...
						if (!m_pReqSharedFile)
						{
						//	If we've just started a download we may want to use that client as a source.
						//	Moreover we need to check if the requested file exists in the DL queue
						//	in order to filter the clients correct way.

						//	Look for the requested file in the download queue
							CPartFile		*pPartFile = g_eMuleApp.m_pDownloadQueue->GetFileByID(requestedFileHash);

						//	If we found the file and it isn't completed...
							if (pPartFile != NULL && pPartFile->IsPartFile())
							{
							//	... and we don't have enough sources already
								if (g_eMuleApp.m_pGlobPrefs->GetMaxSourcePerFile() > pPartFile->GetSourceCount())
								{
									g_eMuleApp.m_pDownloadQueue->CheckAndAddKnownSource(pPartFile, m_pClient);
								}
							}
						//	If we're not currently downloading the requested file...
							else
							{
								m_pClient->UpdateFailedFileRequests();
							}

						//	If the client is using our user hash (hash stealer)
							if (md4cmp(g_eMuleApp.m_pGlobPrefs->GetUserHash(), m_pClient->GetUserHash()) == 0)
								AddDebugLogLine(RGB_LOG_DIMMED + _T("Client %s (%s) (IP: %s) is using our userhash"), m_pClient->GetUserName(), m_pClient->GetClientNameAndVersionString(), m_pClient->GetFullIP());

						}
					//	If we are sharing the requested file...
						else
						{
						//	... and we're downloading the file... (this could be a new source)
							if (m_pReqSharedFile->IsPartFile())
							{
							//	... and we don't have enough sources already
								if (g_eMuleApp.m_pGlobPrefs->GetMaxSourcePerFile() > static_cast<CPartFile*>(m_pReqSharedFile)->GetSourceCount())
								{
									g_eMuleApp.m_pDownloadQueue->CheckAndAddKnownSource((CPartFile*)m_pReqSharedFile, m_pClient);
								}
							}

						//	If this client is requesting this file for the first time
							if (md4cmp(m_pClient->m_reqFileHash, pcPacketBuf) != 0)
							{
								m_pClient->SetCommentDirty();
							}
						//	Send filename etc
							m_pClient->ResetFailedFileRequests();
							m_pClient->SetUploadFileID(reinterpret_cast<BYTE*>(pcPacketBuf));
						//	TODO: Don't let 'ProcessUpFileStatus' re-process the entire pcPacketBuf and search the fileid
						//	again in 'm_pSharedFilesList' -> waste of time.
							m_pClient->ProcessUpFileStatus(pcPacketBuf, dwPacketSize);

							CSafeMemFile		packetStream(128);

							packetStream.Write(m_pReqSharedFile->GetFileHash(), 16);

							uint16		uNameLength = _tcslen(m_pReqSharedFile->GetFileName());

							packetStream.Write(&uNameLength, 2);
							packetStream.Write(m_pReqSharedFile->GetFileName(), uNameLength);

							Packet		*pcPacketBuf = new Packet(&packetStream);

							pcPacketBuf->m_eOpcode = OP_REQFILENAMEANSWER;
							g_eMuleApp.m_pUploadQueue->AddUpDataOverheadFileRequest(pcPacketBuf->m_dwSize);
							SendPacket(pcPacketBuf, true);
							m_pClient->SendCommentInfo(m_pReqSharedFile);
						}
					}
				//	If the packet is too small (malformed)...
					else
					{
						m_pClient->UpdateFailedFileRequests();
						throw CString(_T("invalid OP_REQUESTFILENAME packet size (OP_REQUESTFILENAME)"));
					}
					break;
				}
				case OP_FILEREQANSNOFIL:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadFileRequest(dwPacketSize);
					if (dwPacketSize == 16)
					{
					//	If that client doesn't have my file maybe he has another one
						CPartFile* m_pReqPartFile = g_eMuleApp.m_pDownloadQueue->GetFileByID((uchar*)pcPacketBuf);
						if (!m_pReqPartFile)
							break;

					//	Do nothing if no file was requested
						if (!m_pClient->m_pReqPartFile)
							break;

					//	We try to swap to another file ignoring no needed parts files
						if (m_pClient)
							switch (m_pClient->GetDownloadState())
							{
								case DS_CONNECTED:
								case DS_ONQUEUE:
								case DS_NONEEDEDPARTS:
								{
								//	try to switch the sources in case of A4AF
									if (!m_pClient->SwapToAnotherFile(NULL, true))
										g_eMuleApp.m_pDownloadQueue->RemoveSource(m_pClient, true);
									break;
								}
							}
						break;
					}
					throw CString(_T("invalid OP_REQUESTFILENAME packet size (OP_FILEREQANSNOFIL)"));
					break;
				}
				case OP_REQFILENAMEANSWER:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadFileRequest(dwPacketSize);
					m_pClient->ProcessFileInfo(pcPacketBuf, dwPacketSize);
					break;
				}
				case OP_FILESTATUS:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadFileRequest(dwPacketSize);

					CSafeMemFile	packetStream(reinterpret_cast<BYTE*>(pcPacketBuf), dwPacketSize);

					uchar	pcFileHash[16];

					packetStream.Read(pcFileHash, 16);

					CPartFile	*pReqPartFile = g_eMuleApp.m_pDownloadQueue->GetFileByID(pcFileHash);

					m_pClient->ProcessFileStatus(&packetStream, pReqPartFile);
					break;
				}
				case OP_STARTUPLOADREQ:
				// The old eMule clients (<0.26) as well as original eDonkey don't send a hash (dwPacketSize is 0)
				// so we will add client to UL-queue anyway, but inside AddClientToQueue() will be checked
				// if any file was requested before.
				{
					CKnownFile* m_pReqSharedFile = NULL;

				//	count only real upload request
					m_pClient->SetLastUpRequest();
					m_pClient->AddAskedCount();

					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadFileRequest(dwPacketSize);
					if (dwPacketSize == 16)
					{
						uchar requestedFileHash[16];
						md4cpy(requestedFileHash, pcPacketBuf);
					//check if file exits
						m_pReqSharedFile = g_eMuleApp.m_pSharedFilesList->GetFileByID(requestedFileHash);
						if (m_pReqSharedFile)
						{
						//	If we are downloading this file, this could be a new source
							if (m_pReqSharedFile->IsPartFile())
								if (g_eMuleApp.m_pGlobPrefs->GetMaxSourcePerFile() > ((CPartFile*)m_pReqSharedFile)->GetSourceCount()) //<<--
									g_eMuleApp.m_pDownloadQueue->CheckAndAddKnownSource((CPartFile*)m_pReqSharedFile, m_pClient);

						//	Check to see if this is a new file they are asking for
							if (md4cmp(m_pClient->GetUploadFileID(), pcPacketBuf) != 0)
								m_pClient->SetCommentDirty();

							m_pClient->ResetFailedFileRequests();
							m_pClient->SetUploadFileID((uchar*)pcPacketBuf);
							m_pClient->SendCommentInfo(m_pReqSharedFile);
						}
						else
						{
						//	If client still asking for not existing, he is definitely violating eDonkey protocol
							if (!g_eMuleApp.m_pDownloadQueue->GetFileByID(requestedFileHash))
								m_pClient->UpdateFailedFileRequests();

						//	Reply that requested file does not exist
							Packet * replypacket = new Packet(OP_FILEREQANSNOFIL, 16);
							md4cpy(replypacket->m_pcBuffer, pcPacketBuf);
							g_eMuleApp.m_pUploadQueue->AddUpDataOverheadFileRequest(replypacket->m_dwSize);
							SendPacket(replypacket, true);
							break;
						}
					}
					else
					{
						m_pReqSharedFile = g_eMuleApp.m_pSharedFilesList->GetFileByID((uchar*)m_pClient->m_reqFileHash);
					}
				//	count only real file request
					if (m_pReqSharedFile)
					{
						m_pReqSharedFile->statistic.AddRequest();
						if (!m_pClient->IsOnLAN())
							m_pClient->AddRequestCount((uchar*)m_pClient->m_reqFileHash);

					//	add file to queue
						g_eMuleApp.m_pUploadQueue->AddClientToWaitingQueue(m_pClient);
					}
					break;
				}
				case OP_QUEUERANK:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadFileRequest(dwPacketSize);
					uint32 rank;
					memcpy2(&rank, pcPacketBuf, 4);
					m_pClient->SetRemoteQueueRank(rank);
					break;
				}
				case OP_ACCEPTUPLOADREQ:
				{
					enumClientDLState = m_pClient->GetDownloadState();
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadFileRequest(dwPacketSize);
					if (m_pClient->m_pReqPartFile && g_eMuleApp.m_pDownloadQueue->IsInDLQueue(m_pClient->m_pReqPartFile))
					{
					//	If the client is in our download queue and the part file is downloading/waiting an not paused
					//	Ask for next block while downloading
						if (enumClientDLState == DS_DOWNLOADING)
						{
						//	Sometimes remote client sends us OP_ACCEPTUPLOADREQ during DS_DOWNLOADING state
						//	To prevent desynconization of request we will clear the list of pending blocks
							m_pClient->ClearPendingBlocksList();
							m_pClient->SendBlockRequests();
						}
					//	Ask while connecting, but don't ask for paused files
						else if ( (enumClientDLState == DS_CONNECTED || enumClientDLState == DS_ONQUEUE)
								&& !m_pClient->m_pReqPartFile->IsPaused()
								&& (m_pClient->m_pReqPartFile->GetStatus(false) == PS_READY || m_pClient->m_pReqPartFile->GetStatus(false) == PS_EMPTY))
						{
							m_pClient->SetDownloadState(DS_DOWNLOADING);
							m_pClient->SetLastDownPartAsked(0xFFFF);
							m_pClient->SendBlockRequests();
						}
						else
						{
							Packet		*pcPacketBuf = new Packet(OP_CANCELTRANSFER);

							g_eMuleApp.m_pUploadQueue->AddUpDataOverheadOther(pcPacketBuf->m_dwSize);
							m_pClient->m_pRequestSocket->SendPacket(pcPacketBuf, true, true);
							m_pClient->SetDownloadState(DS_ONQUEUE);
						}
					}
					else
						if (!m_pClient->m_pReqPartFile && m_pClient->GetAskedCountDown() == 0)
							throw CString(_T("client sent OP_ACCEPTUPLOADREQ before file was requested"));
						else
							throw CString(_T("wrong fileID sent (OP_ACCEPTUPLOADREQ => !m_pReqPartFile), file: ") + ((m_pClient->m_pReqPartFile != NULL) ? m_pClient->m_pReqPartFile->GetFileName() : _T("NULL")));
					break;
				}
				case OP_REQUESTPARTS:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadFileRequest(dwPacketSize);

					if (dwPacketSize == 40)	// 40 = 16+(3*4)+(3*4)
					{
						Requested_Block_Struct		reqBlock;

						md4cpy(reqBlock.m_fileHash, pcPacketBuf);

						reqBlock.m_dwStartOffset = *reinterpret_cast<uint32*>(pcPacketBuf + 16);
						reqBlock.m_dwEndOffset = *reinterpret_cast<uint32*>(pcPacketBuf + 16 + 3*4);
						if ((reqBlock.m_dwEndOffset - reqBlock.m_dwStartOffset) != 0)
						{
							m_pClient->AddReqBlock(&reqBlock);
						}

						reqBlock.m_dwStartOffset = *reinterpret_cast<uint32*>(pcPacketBuf + 16 + 4);
						reqBlock.m_dwEndOffset = *reinterpret_cast<uint32*>(pcPacketBuf + 16 + 3*4 + 4);
						if ((reqBlock.m_dwEndOffset - reqBlock.m_dwStartOffset) != 0)
						{
							m_pClient->AddReqBlock(&reqBlock);
						}

						reqBlock.m_dwStartOffset = *reinterpret_cast<uint32*>(pcPacketBuf + 16 + 2*4);
						reqBlock.m_dwEndOffset = *reinterpret_cast<uint32*>(pcPacketBuf + 16 + 3*4 + 2*4);
						if ((reqBlock.m_dwEndOffset - reqBlock.m_dwStartOffset) != 0)
						{
							m_pClient->AddReqBlock(&reqBlock);
						}
					}
					break;
				}
				case OP_CANCELTRANSFER:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadFileRequest(dwPacketSize);
					g_eMuleApp.m_pUploadQueue->RemoveFromUploadQueue(m_pClient, ETS_CANCELED);
					if (g_eMuleApp.m_pGlobPrefs->IsClientTransferLogEnabled())
						g_eMuleApp.m_pdlgEmule->AddDebugLogLine(_T("Client '%s' (%s): upload session ended due to a cancelled transfer."), m_pClient->GetUserName(), m_pClient->GetClientNameAndVersionString());
					m_pClient->SetUploadFileID(NULL);
					break;
				}
				case OP_END_OF_DOWNLOAD:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadFileRequest(dwPacketSize);
					if (dwPacketSize >= 16 && !md4cmp(m_pClient->GetUploadFileID(), pcPacketBuf))
					{
						g_eMuleApp.m_pUploadQueue->RemoveFromUploadQueue(m_pClient, ETS_END_OF_DOWNLOAD);
						if (g_eMuleApp.m_pGlobPrefs->IsClientTransferLogEnabled())
							AddDebugLogLine(_T("Client '%s' (%s): upload session ended due to a completed transfer."), m_pClient->GetUserName(), m_pClient->GetClientNameAndVersionString());
						m_pClient->SetUploadFileID(NULL);
					}
					break;
				}
				case OP_HASHSETREQUEST:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadFileRequest(dwPacketSize);
					if (dwPacketSize != 16)
						throw CString(_T("invalid OP_HASHSETREQUEST packet size"));
					m_pClient->SendHashsetPacket(pcPacketBuf);
					break;
				}
				case OP_HASHSETANSWER:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadFileRequest(dwPacketSize);
					m_pClient->ProcessHashSet(pcPacketBuf, dwPacketSize);
					break;
				}
				case OP_SENDINGPART:
				{
					m_pClient->UpdateLastBlockReceivedTime();
				//	Before we process the packet we need to check the client's download state.
				//	If the source sends us a packet while we're in an unwanted state (we
				//	haven't made a request) we switch him to an error state.
					enumClientDLState = m_pClient->GetDownloadState();
				//	If we're uploading to this client or there are already too many connections...
					if (enumClientDLState == DS_DOWNLOADING || enumClientDLState == DS_TOOMANYCONNS)
					{
						EnumPartFileStatuses	eFileStatus;

					//	If we have made a part file request of this client and the requested file is Waiting...
						if ( (m_pClient->m_pReqPartFile != NULL) &&
							(((eFileStatus = m_pClient->m_pReqPartFile->GetStatus()) == PS_READY) || (eFileStatus == PS_EMPTY)) )
						{
							if (m_pClient->ProcessBlockPacket(pcPacketBuf, dwPacketSize) == 0)
							{
							//	If the file is now in a stopped state (paused/stopped/error)
								if ( (m_pClient->m_pReqPartFile != NULL) &&
									( ((eFileStatus = m_pClient->m_pReqPartFile->GetStatus()) == PS_PAUSED) ||
									(eFileStatus == PS_STOPPED) || (eFileStatus == PS_ERROR) ) )
								{
									Packet		*pPacket = new Packet(OP_CANCELTRANSFER);

									g_eMuleApp.m_pUploadQueue->AddUpDataOverheadOther(pPacket->m_dwSize);
									m_pClient->m_pRequestSocket->SendPacket(pPacket, true, true);
									m_pClient->SetDownloadState((m_pClient->m_pReqPartFile->IsStopped()) ? DS_NONE : DS_ONQUEUE);
								}
							}
							else
							{
							//	Client deletion was requested
								delete m_pClient;
								m_pClient = NULL;
							}
						}
					//	If we didn't want this file (or don't now)
						else
						{
							Packet		*pPacket = new Packet(OP_CANCELTRANSFER);

							g_eMuleApp.m_pUploadQueue->AddUpDataOverheadOther(pPacket->m_dwSize);
							m_pClient->m_pRequestSocket->SendPacket(pPacket, true, true);
							m_pClient->SetDownloadState(((m_pClient->m_pReqPartFile == NULL) || m_pClient->m_pReqPartFile->IsStopped()) ? DS_NONE : DS_ONQUEUE);
						}
					}
					else
						m_pClient->SetDownloadState(DS_ERROR);
					break;
				}
				case OP_OUTOFPARTREQS:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadFileRequest(dwPacketSize);
					if (m_pClient->GetDownloadState() == DS_DOWNLOADING)
					{
						m_pClient->SetDownloadState(DS_ONQUEUE);
					}
					break;
				}
				case OP_SETREQFILEID:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadFileRequest(dwPacketSize);

					if (m_pClient->GetFailedFileRequests() >= 3 && g_eMuleApp.m_pGlobPrefs->IsScanFilterEnabled())
					{
						g_eMuleApp.m_pIPFilter->AddTemporaryBannedIP(m_pClient->GetIP());
						if (!g_eMuleApp.m_pGlobPrefs->IsCMNotLog())
							AddDebugLogLine( RGB_LOG_DIMMED + _T("Client '%s' (%s) added to filtered clients due to file scanning"),
											 m_pClient->GetFullIP(), m_pClient->GetClientNameAndVersionString() );
						Disconnect();
						break;
					}

					if (dwPacketSize == 16)
					{

						CKnownFile* m_pReqSharedFile = g_eMuleApp.m_pSharedFilesList->GetFileByID((uchar*)pcPacketBuf);

						if (!m_pReqSharedFile)
						{
						//	If we just started a download we may want to use that client as a source.
						//  Moreover we need to check if the requested file exists in the DL queue
						//  in order to filter the clients the correct way.
							CPartFile* pPartFile = g_eMuleApp.m_pDownloadQueue->GetFileByID((uchar*)pcPacketBuf);
							if (pPartFile && pPartFile->IsPartFile())
							{
								if (g_eMuleApp.m_pGlobPrefs->GetMaxSourcePerFile() > pPartFile->GetSourceCount()) //<<--
									g_eMuleApp.m_pDownloadQueue->CheckAndAddKnownSource(pPartFile, m_pClient);
							}
							else
								m_pClient->UpdateFailedFileRequests();

						//	Send file request no such file pcPacketBuf (0x48)
							Packet* replypacket = new Packet(OP_FILEREQANSNOFIL, 16);
							md4cpy(replypacket->m_pcBuffer, pcPacketBuf);
							g_eMuleApp.m_pUploadQueue->AddUpDataOverheadFileRequest(replypacket->m_dwSize);
							SendPacket(replypacket, true);
							break;
						}

					//	Reset Failed Request Counter
						m_pClient->ResetFailedFileRequests();
					//	If we are downloading this file, this could be a new source
						if (m_pReqSharedFile->IsPartFile())
							if (g_eMuleApp.m_pGlobPrefs->GetMaxSourcePerFile() > ((CPartFile*)m_pReqSharedFile)->GetSourceCount())
								g_eMuleApp.m_pDownloadQueue->CheckAndAddKnownSource((CPartFile*)m_pReqSharedFile, m_pClient);

					//	Check to see if this is a new file they are asking for
						if (md4cmp(m_pClient->GetUploadFileID(), pcPacketBuf) != 0)
							m_pClient->SetCommentDirty();

					//	Send filestatus
						m_pClient->SetUploadFileID((uchar*)pcPacketBuf);
						CSafeMemFile packetStream(16 + 16);
						packetStream.Write(m_pReqSharedFile->GetFileHash(), 16);
						if (m_pReqSharedFile->IsPartFile())
							((CPartFile*)m_pReqSharedFile)->WritePartStatus(&packetStream);

						else if (m_pReqSharedFile->GetJumpstartEnabled())
							m_pReqSharedFile->WriteJumpstartPartStatus(m_pClient, &packetStream);

						else if (m_pReqSharedFile->HasHiddenParts())
							m_pReqSharedFile->WritePartStatus(&packetStream);

						else
						{
							uint32 null = 0;
							packetStream.Write(&null, 3);
						}

						Packet* pcPacketBuf = new Packet(&packetStream);
						pcPacketBuf->m_eOpcode = OP_FILESTATUS;
						g_eMuleApp.m_pUploadQueue->AddUpDataOverheadFileRequest(pcPacketBuf->m_dwSize);
						SendPacket(pcPacketBuf, true);
					}
					else
						throw CString(_T("invalid OP_REQUESTFILENAME packet size (OP_SETREQFILEID)"));
					break;
				}
				case OP_CHANGE_CLIENT_ID:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(dwPacketSize);
					CSafeMemFile packetStream((BYTE*)pcPacketBuf, dwPacketSize);
					uint32 nNewUserID;
					packetStream.Read(&nNewUserID, 4);
					uint32 nNewServerIP;
					packetStream.Read(&nNewServerIP, 4);
				//	Client changed server and got a LowID
					if (nNewUserID < 0x1000000)
					{
						CServer * pNewServer = g_eMuleApp.m_pServerList->GetServerByIP(nNewServerIP);
						if (pNewServer != NULL)
						{
						// 	Update UserID only if we know the server
							m_pClient->SetUserID(nNewUserID);
							m_pClient->SetServerIP(nNewServerIP);
							m_pClient->SetServerPort(pNewServer->GetPort());
						}
					}
 					//	Client changed server and got a HighID(IP)
 	 					else if (nNewUserID == m_pClient->GetIP())
					{
						m_pClient->SetUserID(nNewUserID);
						CServer* pNewServer = g_eMuleApp.m_pServerList->GetServerByIP(nNewServerIP);
						if (pNewServer != NULL)
						{
							m_pClient->SetServerIP(nNewServerIP);
							m_pClient->SetServerPort(pNewServer->GetPort());
						}
					}
					break;
				}
			//	Sometimes sent by Hybrid
				case OP_CHANGE_SLOT:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(dwPacketSize);
					break;
				}
				case OP_MESSAGE:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(dwPacketSize);

					uint16		uMessageLen;
					memcpy2(&uMessageLen, pcPacketBuf, 2);

				//	Verify the packet size
					if (uMessageLen + 2 != dwPacketSize)
						throw CString(_T("invalid message packet (OP_MESSAGE)"));

					char		*strMessage = new char[uMessageLen + 1];

					memcpy2(strMessage, pcPacketBuf + 2, uMessageLen);
					strMessage[uMessageLen] = 0;
					g_eMuleApp.m_pdlgEmule->m_wndChat.m_ctlChatSelector.ProcessMessage(m_pClient, strMessage);
					delete[] strMessage;
					break;
				}
				case OP_ASKSHAREDFILES:
				{
				//	Client wants to know what we have in share, let's see if we allow him to know that
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(dwPacketSize);

				//	IP banned, no answer for this request
					if (m_pClient->IsBanned())
					{
						break;
					}

					CPtrList	list;

					if ( g_eMuleApp.m_pGlobPrefs->CanSeeShares() == SEE_SHARE_EVERYBODY
					  || (g_eMuleApp.m_pGlobPrefs->CanSeeShares() == SEE_SHARE_FRIENDS && m_pClient->IsFriend()) )
					{
						CCKey		bufKey;
						CKnownFile	*cur_file;

						for (POSITION pos = g_eMuleApp.m_pSharedFilesList->m_mapSharedFiles.GetStartPosition(); pos != NULL;)
						{
							g_eMuleApp.m_pSharedFilesList->m_mapSharedFiles.GetNextAssoc(pos, bufKey, cur_file);
							list.AddTail((void*&)cur_file);
						}
						AddLogLine( true, RGB_LOG_NOTICE + GetResString(IDS_REQ_SHAREDFILES),
										 m_pClient->GetUserName(), m_pClient->GetClientNameAndVersionString(), GetResString(IDS_ACCEPTED) );
					}
					else
					{
						AddLogLine( true, RGB_LOG_WARNING + GetResString(IDS_REQ_SHAREDFILES),
										 m_pClient->GetUserName(), m_pClient->GetClientNameAndVersionString(), GetResString(IDS_DENIED) );
					}

				//	Build the reply packet and send it.
					uint32			dwNumFiles = list.GetCount();
					CSafeMemFile	packetStream(80);

					packetStream.Write(&dwNumFiles, 4);
					while (list.GetCount())
					{
						g_eMuleApp.m_pSharedFilesList->WriteToOfferedFilePacket(*static_cast<CKnownFile*>(list.GetHead()), packetStream, NULL);
						list.RemoveHead();
					}

					Packet		*pReplyPacket = new Packet(&packetStream);

					pReplyPacket->m_eOpcode = OP_ASKSHAREDFILESANSWER;
					g_eMuleApp.m_pUploadQueue->AddUpDataOverheadOther(pReplyPacket->m_dwSize);
					SendPacket(pReplyPacket, true, true);
					break;
				}
				case OP_ASKSHAREDFILESANSWER:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(dwPacketSize);
					m_pClient->ProcessSharedFileList(pcPacketBuf, dwPacketSize);
					break;
				}
				case OP_ASKSHAREDDIRS:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(dwPacketSize);
					ASSERT(dwPacketSize == 0);

				//	IP banned, no answer for this request
					if (m_pClient->IsBanned())
					{
						break;
					}

					if ( g_eMuleApp.m_pGlobPrefs->CanSeeShares() == SEE_SHARE_EVERYBODY
					  || (g_eMuleApp.m_pGlobPrefs->CanSeeShares() == SEE_SHARE_FRIENDS && m_pClient->IsFriend()) )
					{
						AddLogLine( true, RGB_LOG_NOTICE + GetResString(IDS_SHAREDREQ1), m_pClient->GetUserName(),
							m_pClient->GetClientNameAndVersionString(), GetResString(IDS_ACCEPTED) );

						CString		strDir, strTest;

					//	Virtual folder list
						CArray<CString, CString&>	astrFolders;
						CMapStringToString			&mapVDirs = g_eMuleApp.m_pSharedFilesList->GetSharedVDirForList();
						POSITION					pos = mapVDirs.GetStartPosition();

						while (pos)
						{
							CString		strKey, strDir;

							mapVDirs.GetNextAssoc(pos, strKey, strDir);

							int			i = 0;

							while ((i < astrFolders.GetCount()) && (strKey.CompareNoCase(astrFolders.GetAt(i)) > 0))
								i++;
							astrFolders.InsertAt(i, strKey);
						}

					//	Build the reply packet
						CSafeMemFile		packetStream(80);
						uint32				dwNumDirs = astrFolders.GetCount();

						packetStream.Write(&dwNumDirs, 4);
					//	For each directory in the array...
						for (int ix = 0; ix < astrFolders.GetCount(); ix++)
						{
							const CString		&strDir = astrFolders.GetAt(ix);
							uint16				uCnt = strDir.GetLength();

							packetStream.Write(&uCnt, 2);
							packetStream.Write(static_cast<LPCTSTR>(strDir), uCnt);
						}

						Packet		*pReplyPacket = new Packet(&packetStream);

						pReplyPacket->m_eOpcode = OP_ASKSHAREDDIRSANS;
						g_eMuleApp.m_pUploadQueue->AddUpDataOverheadOther(pReplyPacket->m_dwSize);
						SendPacket(pReplyPacket, true, true);
					}
					else
					{
						AddLogLine( true, RGB_LOG_WARNING + GetResString(IDS_SHAREDREQ1), m_pClient->GetUserName(),
							m_pClient->GetClientNameAndVersionString(), GetResString(IDS_DENIED) );

						Packet		*pReplyPacket = new Packet(OP_ASKSHAREDDENIEDANS);

						g_eMuleApp.m_pUploadQueue->AddUpDataOverheadOther(pReplyPacket->m_dwSize);
						SendPacket(pReplyPacket, true, true);
					}
					break;
				}
				case OP_ASKSHAREDFILESDIR:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(dwPacketSize);

				//	IP banned, no answer for this request
					if (m_pClient->IsBanned())
					{
						break;
					}

					CSafeMemFile		packetStream(reinterpret_cast<uchar*>(pcPacketBuf), dwPacketSize);

					uint16		uDirNameLen;
					packetStream.Read(&uDirNameLen, 2);

					CString		strReqDir, strReqVDir;
					packetStream.Read(strReqVDir.GetBuffer(uDirNameLen), uDirNameLen);
					strReqVDir.ReleaseBuffer(uDirNameLen);
					PathRemoveBackslash(strReqVDir.GetBuffer());
					strReqVDir.ReleaseBuffer();

				//	Retrieve real dir from virtual dir
					CMapStringToString		&mapVDirs = g_eMuleApp.m_pSharedFilesList->GetSharedVDirForList();

					bool		bReqVDir = mapVDirs.Lookup(strReqVDir, strReqDir);

					if ( (bReqVDir)
					  && ( g_eMuleApp.m_pGlobPrefs->CanSeeShares() == SEE_SHARE_EVERYBODY
					    || (g_eMuleApp.m_pGlobPrefs->CanSeeShares() == SEE_SHARE_FRIENDS && m_pClient->IsFriend()) ) )
					{
						AddDebugLogLine( _T("Client '%s' (%s) requested your list of shared files for directory '%s' ['%s'] (accepted)"),
										 m_pClient->GetUserName(), m_pClient->GetClientNameAndVersionString(), strReqVDir, strReqDir );
						ASSERT(packetStream.GetPosition() == packetStream.GetLength());

						CCKey		bufKey;
						CKnownFile	*pKnownFile;

						CTypedPtrList<CPtrList, CKnownFile*>	list;

						for (POSITION pos = g_eMuleApp.m_pSharedFilesList->m_mapSharedFiles.GetStartPosition(); pos != NULL;)
						{
							g_eMuleApp.m_pSharedFilesList->m_mapSharedFiles.GetNextAssoc(pos, bufKey, pKnownFile);

							CString		strSharedFileDir(pKnownFile->GetPath());

							PathRemoveBackslash(strSharedFileDir.GetBuffer());
							strSharedFileDir.ReleaseBuffer();
							if (strReqDir.CompareNoCase(strSharedFileDir) == 0)
								list.AddTail(pKnownFile);
						}

					//	Currently we are sending each shared directory, even if it does not contain any files.
					//	Because of this we also have to send an empty shared files list..
						CSafeMemFile		packetStream(80);

						uint16		uDirNameLen = strReqVDir.GetLength();
						packetStream.Write(&uDirNameLen, 2);

						packetStream.Write(static_cast<LPCTSTR>(strReqVDir), uDirNameLen);

						uint32		dwNumFiles = list.GetCount();
						packetStream.Write(&dwNumFiles, 4);

						while (list.GetCount())
						{
							g_eMuleApp.m_pSharedFilesList->WriteToOfferedFilePacket(*list.GetHead(), packetStream, NULL);
							list.RemoveHead();
						}

						Packet		*pReplyPacket = new Packet(&packetStream);

						pReplyPacket->m_eOpcode = OP_ASKSHAREDFILESDIRANS;
						g_eMuleApp.m_pUploadQueue->AddUpDataOverheadOther(pReplyPacket->m_dwSize);
						SendPacket(pReplyPacket, true, true);
					}
				//	If the requester isn't allowed to see our shares...
					else
					{
						AddDebugLogLine( _T("Client '%s' (%s) requested your list of shared files for directory '%s' ['%s'] (denied)"),
										 m_pClient->GetUserName(), m_pClient->GetClientNameAndVersionString(), strReqVDir, strReqDir );

						Packet		*pReplyPacket = new Packet(OP_ASKSHAREDDENIEDANS);

						g_eMuleApp.m_pUploadQueue->AddUpDataOverheadOther(pReplyPacket->m_dwSize);
						SendPacket(pReplyPacket, true, true);
					}
					break;
				}
				case OP_ASKSHAREDDIRSANS:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(dwPacketSize);
					if (m_pClient->GetFileListRequested() == 1)
					{
						CSafeMemFile		packetStream(reinterpret_cast<uchar*>(pcPacketBuf), dwPacketSize);

						uint32		dwNumDirs;
						packetStream.Read(&dwNumDirs, 4);

						for (UINT i = 0; i < dwNumDirs; i++)
						{
							uint16		uDirNameLen;
							packetStream.Read(&uDirNameLen, 2);

							CString		strDirName;
							packetStream.Read(strDirName.GetBuffer(uDirNameLen), uDirNameLen);
							strDirName.ReleaseBuffer(uDirNameLen);

							AddDebugLogLine( _T("Client '%s' (%s) shares directory '%s'"),
											 m_pClient->GetUserName(), m_pClient->GetClientNameAndVersionString(), strDirName );

							CSafeMemFile		packetStream2(80);

							uDirNameLen = strDirName.GetLength();
							packetStream2.Write(&uDirNameLen, 2);
							packetStream2.Write(static_cast<LPCTSTR>(strDirName), uDirNameLen);

							Packet		*pReplyPacket = new Packet(&packetStream2);

							pReplyPacket->m_eOpcode = OP_ASKSHAREDFILESDIR;
							g_eMuleApp.m_pUploadQueue->AddUpDataOverheadOther(pReplyPacket->m_dwSize);
							SendPacket(pReplyPacket, true, true);
						}

						ASSERT(packetStream.GetPosition() == packetStream.GetLength());
						m_pClient->SetFileListRequested(dwNumDirs);
					}
					else
					{
						AddDebugLogLine( _T("Client '%s' (%s) sent not requested list of shared directories - ignored"),
										 m_pClient->GetUserName(), m_pClient->GetClientNameAndVersionString() );
					}

					break;
				}
				case OP_ASKSHAREDFILESDIRANS:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(dwPacketSize);

					CSafeMemFile		packetStream(reinterpret_cast<uchar*>(pcPacketBuf), dwPacketSize);

				//	Get the directory name length
					uint16		uDirNameLen;
					packetStream.Read(&uDirNameLen, 2);

				//	Get the directory name
					CString		strDirName;
					packetStream.Read(strDirName.GetBuffer(uDirNameLen), uDirNameLen);
					strDirName.ReleaseBuffer(uDirNameLen);
					PathRemoveBackslash(strDirName.GetBuffer());
					strDirName.ReleaseBuffer();

					if (m_pClient->GetFileListRequested() > 0)
					{
						AddDebugLogLine( _T("Client '%s' (%s) sent list of shared files for directory '%s'"),
										 m_pClient->GetUserName(), m_pClient->GetClientNameAndVersionString(), strDirName );
						m_pClient->ProcessSharedFileList(pcPacketBuf + packetStream.GetPosition(), dwPacketSize - packetStream.GetPosition(), strDirName);
						if (m_pClient->GetFileListRequested() == 0)
						{
							AddDebugLogLine( _T("Client '%s' (%s) finished sending list of shared files"),
											 m_pClient->GetUserName(), m_pClient->GetClientNameAndVersionString() );
						}
					}
					else
					{
						AddDebugLogLine( _T("Client '%s' (%s) sent not requested list of shared files for directory '%s' - ignored"),
										 m_pClient->GetUserName(), m_pClient->GetClientNameAndVersionString(), strDirName);
					}
					break;
				}
				case OP_ASKSHAREDDENIEDANS:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(dwPacketSize);
					ASSERT(dwPacketSize == 0);
					AddLogLine(true, IDS_SHAREDREQDENIED, m_pClient->GetUserName(), m_pClient->GetClientNameAndVersionString());
					m_pClient->SetFileListRequested(0);
					break;
				}
				case OP_UNKNOWNx6A:
				case OP_UNKNOWNx7A:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(dwPacketSize);
					break;
				}
				default:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(dwPacketSize);
				}
			}
		}
		catch (CFileException * error)
		{
			OUTPUT_DEBUG_TRACE();
			error->Delete();
			throw CString(_T("invalid or corrupted packet received"));
		}
		catch (CMemoryException * error)
		{
			OUTPUT_DEBUG_TRACE();
			error->Delete();
			throw CString(_T("Memory exception"));
		}
	}
	catch (CString error)
	{
		OUTPUT_DEBUG_TRACE();
		if (m_pClient)
		{
			m_pClient->SetDownloadState(DS_ERROR);
			AddDebugLogLine( _T("Client '%s' (%s) caused an error: %s. Disconnecting client!"),
							 m_pClient->GetUserName(), m_pClient->GetClientNameAndVersionString(),
						 	 error.GetBuffer() );
		}

		Disconnect();
		return false;
	}
	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CClientReqSocket::ProcessExtPacket(char* packet, uint32 size, EnumOpcodes opcode)
{
	try
	{
		try
		{
			if (!m_pClient)
				throw CString(_T("unknown client sends extended protocol packet"));

			EnumDLQState enumClientDLState = m_pClient->GetDownloadState();

			switch (opcode)
			{
				case OP_EMULEINFO:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(size);
					m_pClient->ProcessMuleInfoPacket(reinterpret_cast<BYTE*>(packet), size);
					m_pClient->SendMuleInfoPacket(true);
					break;
				}
				case OP_EMULEINFOANSWER:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(size);
					m_pClient->ProcessMuleInfoPacket(reinterpret_cast<BYTE*>(packet), size);
					break;
				}
				case OP_SECIDENTSTATE:
				{
					m_pClient->ProcessSecIdentStatePacket(reinterpret_cast<BYTE*>(packet), size);
					if (m_pClient->GetSecureIdentState() == IS_SIGNATURENEEDED)
					{
						m_pClient->SendSignaturePacket();
					}
					else if (m_pClient->GetSecureIdentState() == IS_KEYANDSIGNEEDED)
					{
						m_pClient->SendPublicKeyPacket();
						m_pClient->SendSignaturePacket();
					}
					break;
				}
				case OP_PUBLICKEY:
				{
					if (!m_pClient->IsBanned())
					{
						m_pClient->ProcessPublicKeyPacket((uchar*)packet, size);
					}
					break;
				}
				case OP_SIGNATURE:
				{
					if (!m_pClient->IsBanned())
					{
						m_pClient->ProcessSignaturePacket((uchar*)packet, size);
					}
					break;
				}

				case OP_COMPRESSEDPART:
				{
					m_pClient->UpdateLastBlockReceivedTime();
				//	Before we process a packet we need to check Download State
				//	If client send us a packet in an unwanted state (we don't request (need) this packet)
				//	we will switch him to error state.
					if (enumClientDLState == DS_DOWNLOADING || enumClientDLState == DS_TOOMANYCONNS)
					{
						EnumPartFileStatuses	eFileStatus;

						if ( (m_pClient->m_pReqPartFile != NULL) &&
							(((eFileStatus = m_pClient->m_pReqPartFile->GetStatus()) == PS_READY) || (eFileStatus == PS_EMPTY)) )
						{
							m_pClient->ProcessBlockPacket(packet, size, true);
							if ( (m_pClient->m_pReqPartFile != NULL) &&
								( ((eFileStatus = m_pClient->m_pReqPartFile->GetStatus()) == PS_PAUSED) ||
								(eFileStatus == PS_STOPPED) || (eFileStatus == PS_ERROR) ) )
							{
								Packet * packet = new Packet(OP_CANCELTRANSFER);
								g_eMuleApp.m_pUploadQueue->AddUpDataOverheadOther(packet->m_dwSize);
								m_pClient->m_pRequestSocket->SendPacket(packet, true, true);
								m_pClient->SetDownloadState((m_pClient->m_pReqPartFile->IsStopped()) ? DS_NONE : DS_ONQUEUE);
							}
						}
						else
						{
							Packet* packet = new Packet(OP_CANCELTRANSFER);
							g_eMuleApp.m_pUploadQueue->AddUpDataOverheadOther(packet->m_dwSize);
							m_pClient->m_pRequestSocket->SendPacket(packet, true, true);
							m_pClient->SetDownloadState(((m_pClient->m_pReqPartFile == NULL) || m_pClient->m_pReqPartFile->IsStopped()) ? DS_NONE : DS_ONQUEUE);
						}
					}
					else
						m_pClient->SetDownloadState(DS_ERROR);
					break;
				}
				case OP_QUEUERANKING:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(size);
					if (size != 12)
						throw CString(_T("invalid size (OP_QUEUERANKING)"));
					uint16 newrank;
					memcpy2(&newrank, packet, 2);
					m_pClient->SetRemoteQueueFull(false);
					m_pClient->SetRemoteQueueRank(newrank);
				//	Switch DL state in case newrank != 0
					if (newrank && enumClientDLState == DS_DOWNLOADING)
						m_pClient->SetDownloadState(DS_ONQUEUE);
					break;
				}
				case OP_REQUESTSOURCES:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadSourceExchange(size);
				//	Original client don't answer if client's SourceExchange Version is less or equal than 1
				//	but i think we need to keep compatibility with old ePlus versions
					if (m_pClient->GetSourceExchangeVersion() >= 1)
					{
						if (size != 16)
							throw CString(_T("invalid size (OP_REQUESTSOURCES)"));

					//	First check shared file list, then download list
						CKnownFile	   *file = g_eMuleApp.m_pSharedFilesList->GetFileByID((uchar*)packet);

						if (!file)
							file = g_eMuleApp.m_pDownloadQueue->GetFileByID((uchar*)packet);

						if (file)
						{
							DWORD		dwTimePassed = ::GetTickCount() - m_pClient->GetLastSrcReqTime() + CONNECTION_LATENCY;
							bool		bNeverAskedBefore = m_pClient->GetLastSrcReqTime() == 0;

							if (
							//	If not complete and file is rare, allow once every 10 minutes
							    ( file->IsPartFile()
							      && ((CPartFile*)file)->GetSourceCount() <= RARE_FILE * 2
							      && (bNeverAskedBefore || dwTimePassed > SOURCECLIENTREASK) )
							//	OR if file is not rare or if file is complete, allow every 90 minutes
							    || (bNeverAskedBefore || dwTimePassed > SOURCECLIENTREASK * MINCOMMONPENALTY) )
							{

								m_pClient->SetLastSrcReqTime();

								Packet	   *tosend = file->CreateSrcInfoPacket(m_pClient);

								if (tosend)
								{
									g_eMuleApp.m_pUploadQueue->AddUpDataOverheadSourceExchange(tosend->m_dwSize);
									SendPacket(tosend, true, true);
								}
							}
						}
					}
					break;
				}
				case OP_ANSWERSOURCES:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadSourceExchange(size);

					try
					{
						CSafeMemFile		packetStream(reinterpret_cast<BYTE*>(packet), size);

						uchar				hash[16];

						packetStream.Read(hash, 16);

						CKnownFile			*pKnownFile = g_eMuleApp.m_pDownloadQueue->GetFileByID(reinterpret_cast<BYTE*>(packet));

						if (pKnownFile != NULL)
						{
							if (pKnownFile->IsPartFile())
							{
							//	Set the client's answer time
								m_pClient->SetLastSrcAnswerTime();
							//	and set the pKnownFile's last answer time
								((CPartFile*)pKnownFile)->SetLastAnsweredTime();
							//	Support Source Exchange v2
								((CPartFile*)pKnownFile)->AddClientSources(&packetStream, m_pClient->GetSourceExchangeVersion());
							}
						}
					}
					catch (CFileException * error)
					{
						OUTPUT_DEBUG_TRACE();
						error->Delete();
						throw CString(_T("invalid or corrupted packet received"));
					}
					break;
				}
				case OP_FILEDESC:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(size);
					m_pClient->ProcessMuleCommentPacket(packet, size);
					break;
				}
/*
				case OP_VERIFYUPSREQ:
				{
					CUpDownClient* vuclient = g_eMuleApp.m_pClientList->VUGetRandomClient();
					if (vuclient)
					{
						Packet* answer = new Packet(OP_VERIFYUPSANSWER,6,OP_EMULEPROT);
						uint32 ip = vuclient->GetIP();
						memcpy2(answer->m_pcBuffer,ip,4);
						uint16 udpport = vuclient->GetUDPPort();
						memcpy2(answer->m_pcBuffer,ip,4);
						g_eMuleApp.m_pUploadQueue->AddUpDataRateMSOverhead(packet->m_dwSize);
						SendPacket(answer,true,true);
					}
				}
				case OP_VERIFYUPSANSWER:
				{
					if (size != 12)
						throw CString("invalid size (OP_VERIFYUPSANSWER)");
				}
*/
				default:
				{
					g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(size);
				}
			}
		}
		catch (CFileException *error)
		{
			OUTPUT_DEBUG_TRACE();
			error->Delete();
			throw CString(_T("invalid or corrupted packet received"));
		}
		catch(CMemoryException *error)
		{
			OUTPUT_DEBUG_TRACE();
			error->Delete();
			throw CString(_T("Memory exception"));
		}
	}
	catch (CString error)
	{
		OUTPUT_DEBUG_TRACE();
		if (!m_pClient)
			AddDebugLogLine(_T("A client caused an error or did something bad: %s. Disconnecting client!"), error.GetBuffer());
		else
			AddDebugLogLine( _T("Client '%s' (%s) caused an error or did something bad: %s. Disconnecting client!"),
							 m_pClient->GetUserName(), m_pClient->GetClientNameAndVersionString(), error.GetBuffer() );
		if (m_pClient)
			m_pClient->SetDownloadState(DS_ERROR);
		Disconnect();
		return false;
	}
	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CClientReqSocket::OnInit()
{
	byte tv = 1;
//SetSockOpt(SO_DONTLINGER,&tv,sizeof(BOOL));
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CClientReqSocket::OnSend(int nErrorCode)
{
	ResetTimeOutTimer();
	CEMSocket::OnSend(nErrorCode);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CClientReqSocket::OnError(int nErrorCode)
{

	if (m_pClient)
		AddDebugLogLine( _T("Client '%s' (%s) (IP: %s) caused an error: %u. Disconnecting client!"),
						 m_pClient->GetUserName(), m_pClient->GetClientNameAndVersionString(), m_pClient->GetFullIP(), nErrorCode );
	else
		AddDebugLogLine(_T("A client caused an error or did something bad: CClientReqSocket::OnError. Disconnecting client!"));
	Disconnect();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CClientReqSocket::PacketReceived(Packet* packet)
{
	try
	{
	//	Filter clients that are violating the protocol
#ifdef OLD_SOCKETS_ENABLED
		SOCKADDR_IN sockAddr;
		memzero(&sockAddr, sizeof(sockAddr));
		uint32 nSockAddrLen = sizeof(sockAddr);
		GetPeerName((SOCKADDR*) & sockAddr, (int*) & nSockAddrLen);
		uint32 UserIP = sockAddr.sin_addr.S_un.S_addr;
		CString FullUserIP(inet_ntoa(sockAddr.sin_addr));
		if (g_eMuleApp.m_pIPFilter->IsFiltered(UserIP))
		{
			InterlockedIncrement(&g_eMuleApp.m_lIncomingFiltered);
			InterlockedIncrement(&g_eMuleApp.m_lTotalFiltered);
		//	Reset client to NULL in order don't let CUpDownClient::Disconnected be processed
		//	for a client which is not in the list of clients
			m_pClient = NULL;
			Disconnect();
			return;
		}
#endif //OLD_SOCKETS_ENABLED

		switch (packet->m_eProtocol)
		{
			case OP_EDONKEYPROT:
				ProcessPacket(packet->m_pcBuffer, packet->m_dwSize, packet->m_eOpcode);
				break;
			case OP_PACKEDPROT:
				if (!packet->UnpackPacket())
				{
					ASSERT (false);
					break;
				}
			case OP_EMULEPROT:
				ProcessExtPacket(packet->m_pcBuffer, packet->m_dwSize, packet->m_eOpcode);
				break;
			default:
			//	Lets free the socket from buggy and unknown clients
				if (m_pClient)
					m_pClient->SetDownloadState(DS_ERROR);
				Disconnect();
		}
	}
	catch(...)
	{
		OUTPUT_DEBUG_TRACE();
	//	If we get here, we probably had an access violation
//		ASSERT(0);
		AddDebugLogLine(_T("%s: Unknown exception. Protocol=0x%02x  opcode=0x%02x  size=%u"), __FUNCTION__, packet != NULL ? packet->m_eProtocol : 0, packet != NULL ? packet->m_eOpcode : 0, packet != NULL ? packet->m_dwSize : 0);

	//	TODO: This exception handler should definitively be *here*. Though we may get some very
	//	strange socket deletion crashs if we disconnect a client's TCP socket on catching an exception
	//	here. See also comments in 'CAsyncSocketExHelperWindow::WindowProc'
	//	if (m_pClient)
	//		m_pClient->SetDownloadState(DS_ERROR);
	//	Disconnect();
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CClientReqSocket::OnReceive(int nErrorCode)
{
	ResetTimeOutTimer();
	CEMSocket::OnReceive(nErrorCode);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CClientReqSocket::Create()
{
	g_eMuleApp.m_pListenSocket->AddConnection();
	BOOL result = CAsyncSocketEx::Create(0, SOCK_STREAM, FD_WRITE | FD_READ | FD_CLOSE); // deadlake PROXYSUPPORT - changed to AsyncSocketEx
	OnInit();
	return result;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//	CListenSocket member functions

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	CListenSocket constructor
CListenSocket::CListenSocket(CPreferences *in_pPrefs)
{
	m_pPrefs = in_pPrefs;
	m_iNumOpenSockets = 0;
	m_iMaxConnectionsReachedCount = 0;
	m_iOpenSocketsInterval = 0;
	InterlockedExchange(&m_lNumPendingConnections, 0);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CListenSocket::~CListenSocket()
{
	Close();
	KillAllSockets();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CListenSocket::StartListening()
{
	m_bListening = true;

	return(this->Create(m_pPrefs->GetPort(), SOCK_STREAM, FD_ACCEPT) && this->Listen());
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CListenSocket::RestartListening()
{
	m_bListening = true;
	if (InterlockedDecrement(&m_lNumPendingConnections) >= 0)
	{
		OnAccept(0);
	}
	else
	{
		InterlockedExchange(&m_lNumPendingConnections, 0);
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CListenSocket::StopListening()
{
	m_bListening = false;
	m_iMaxConnectionsReachedCount++;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	OnAccept() is called to notify the request socket that it can accept pending connection requests by calling
//	Accept().
void CListenSocket::OnAccept(int nErrorCode)
{
	if (nErrorCode == 0)
	{
		InterlockedIncrement(&m_lNumPendingConnections);
	//	If the number of pending connections was less than 0 (which obviously shouldn't happen)...
		if (InterlockedCompareExchange(&m_lNumPendingConnections, 1, 0) == 0)
		{
			//ASSERT(false);
			//m_lNumPendingConnections = 1;
		}
		if (TooManySockets(true) && !g_eMuleApp.m_pServerConnect->IsConnecting())
		{
			StopListening();
			return;
		}
		else if (m_bListening == false)
			RestartListening(); //If the client is still at maxconnections, this will allow it to go above it.. But if you don't, you will get a lowID on all servers.

		while (InterlockedDecrement(&m_lNumPendingConnections) >= 0)
		{
			//m_lNumPendingConnections--;
			CClientReqSocket* newclient = new CClientReqSocket(m_pPrefs);
			if (!Accept(*newclient))
				newclient->Safe_Delete();
			else
			{
				newclient->AsyncSelect(FD_WRITE | FD_READ | FD_CLOSE);
				newclient->OnInit();
			}
			AddConnection();
		}
	//	if (TooManySockets(true) && !g_eMuleApp.m_pServerConnect->IsConnecting())
	//	StopListening();
	}

}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Process() takes care of cleaning up invalid, deleted, and timed out sockets from the open request socket list.
void CListenSocket::Process()
{
	POSITION	pos2;

	m_iOpenSocketsInterval = 0;
	m_iNumOpenSockets = 0;

//	For each open request socket...
	for (POSITION pos1 = m_openSocketList.GetHeadPosition(); (pos2 = pos1) != NULL;)
	{
		m_openSocketList.GetNext(pos1);

		CClientReqSocket	*pSocket = m_openSocketList.GetAt(pos2);

		m_iNumOpenSockets++;

	//	If the request socket is marked for deletion...
		if (pSocket->m_bDeleteThis)
		{
			if (pSocket->m_SocketData.hSocket != INVALID_SOCKET) // deadlake PROXYSUPPORT - changed to AsyncSocketEx
			{
				pSocket->Close();
			}
			else
			{
				pSocket->TimedDelete();
			}
		}
		else
		{
		//	If the request socket has timed out, disconnect it.
			m_openSocketList.GetAt(pos2)->CheckTimeOut();
		}
	}
//	If we're not currently listening for new connections and we're either just connecting to a server
//	or we're not within 5 connections of max then start listening again.
	if ((GetNumOpenSockets() + 5 < m_pPrefs->GetMaxConnections() || g_eMuleApp.m_pServerConnect->IsConnecting()) && !m_bListening)
		RestartListening();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CListenSocket::RecalculateStats()
{
	memzero(m_ConnectionStates, 6);
	POSITION pos1, pos2;
	for (pos1 = m_openSocketList.GetHeadPosition(); (pos2 = pos1) != NULL;)
	{
		m_openSocketList.GetNext(pos1);
		CClientReqSocket* cur_sock = m_openSocketList.GetAt(pos2);
		switch (cur_sock->GetConState())
		{
			case ES_DISCONNECTED:
			m_ConnectionStates[0]++;
			break;
			case ES_NOTCONNECTED:
			m_ConnectionStates[1]++;
			break;
			case ES_CONNECTED:
			m_ConnectionStates[2]++;
			break;
		}
	}
}

void CListenSocket::AddSocket(CClientReqSocket* toadd)
{
	m_openSocketList.AddTail(toadd);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CListenSocket::RemoveSocket(CClientReqSocket *pSocket)
{
	POSITION	pos;

	pos = m_openSocketList.Find(pSocket);
	if (pos != NULL)
		m_openSocketList.RemoveAt(pos);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	DeleteSocket() schedules the socket 'pSocket' for deletion if it's in the open socket list.
bool CListenSocket::DeleteSocket(CClientReqSocket *pSocket)
{
	bool		bDeleted = false;

	if (pSocket != NULL && IsValidSocket(pSocket))
	{
		pSocket->Safe_Delete();
		bDeleted = true;
	}

	return bDeleted;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CListenSocket::KillAllSockets()
{
	for (POSITION pos = m_openSocketList.GetHeadPosition(); pos != NULL; pos = m_openSocketList.GetHeadPosition())
	{
		CClientReqSocket * cur_socket = m_openSocketList.GetAt(pos);
		if (cur_socket->m_pClient)
			delete cur_socket->m_pClient;
		else
			delete cur_socket;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CListenSocket::AddConnection()
{
	m_iOpenSocketsInterval++;
	m_iNumOpenSockets++;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	TooManySockets() returns true if the number of open sockets is greater than the number specified
//	in Preferences. If 'bIgnoreInterval' is false, it will also return true if the number of
//	new connections since the last Process() call is greater than the "connections per 5 secs"
//	setting in Preferences.
bool CListenSocket::TooManySockets(bool bIgnoreInterval)
{
	if ( GetNumOpenSockets() > m_pPrefs->GetMaxConnections()
	     || ( m_iOpenSocketsInterval > (g_eMuleApp.m_pGlobPrefs->GetMaxConPerFive() * g_eMuleApp.m_pdlgEmule->m_dlgStatistics.GetMaxConPerFiveModifier())
	          && !bIgnoreInterval ) )
	{
		return true;
	}
	else
		return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	IsValidSocket() returns true if 'pSocket' is in the open request socket list.
bool CListenSocket::IsValidSocket(CClientReqSocket *pSocket)
{
	return m_openSocketList.Find(pSocket);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CListenSocket::Debug_ClientDeleted(CUpDownClient* deleted)
{
	POSITION pos1, pos2;
	for (pos1 = m_openSocketList.GetHeadPosition();(pos2 = pos1) != NULL;)
	{
		m_openSocketList.GetNext(pos1);
		CClientReqSocket* cur_sock = m_openSocketList.GetAt(pos2);
		if (!AfxIsValidAddress(cur_sock, sizeof(CClientReqSocket)))
		{
			AfxDebugBreak();
		}
		if (cur_sock->m_pClient == deleted)
		{
			AfxDebugBreak();
		}
	}
}
#endif //OLD_SOCKETS_ENABLED
