//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once

#include "types.h"
#include "opcodes.h"
#include "otherstructs.h"
#include "zlib/zlib.h"

//			CLIENT TO SERVER
//			PACKET CLASS
// TODO some parts could need some work to make it more efficient
class Packet
{
public:
					Packet(EnumProtocols eProtocol = OP_EDONKEYPROT);
					Packet(PacketHeader_Struct *pHeaderStruct); // only used for receiving packets
					Packet(CMemFile *pDataFile,EnumProtocols eProtocol = OP_EDONKEYPROT);
				Packet(char* pPacketPart,uint32 nSize,bool bLast, byte byteFilePriority, bool bFromPF = true); // only used for split packets!
					Packet(EnumOpcodes eOpcode,uint32 iSize = 0,EnumProtocols eProtocol = OP_EDONKEYPROT,bool bFromPF = true);
				Packet(EnumOpcodes eOpcode, uint32 iSize, EnumProtocols eProtocol, byte byteFilePriority, bool bFromPF);
				   ~Packet();
	char		   *GetHeader();
	char		   *GetUDPHeader();
	char		   *GetPacket();
	char		   *DetachPacket();
	uint32			GetRealPacketSize()		{return m_dwSize+sizeof(PacketHeader_Struct);}
	bool			IsSplit()			{return m_bSplit;}
	bool			IsLastSplit()		{return m_bLastSplit;}
	bool			IsFromPF()			{return m_bFromPF;}	//	khaos stats
	byte 			GetFilePriority()	{return m_byteFilePriority;}
	void			PackPacket();
	bool			UnpackPacket(uint32 dwMaxUnpackedSize = 0);

	char		   *m_pcBuffer;
	uint32			m_dwSize;
	EnumOpcodes		m_eOpcode;
	EnumProtocols	m_eProtocol;

private:
	byte		m_byteFilePriority;
	bool			m_bSplit;
	bool			m_bLastSplit;
	bool			m_bFromPF;		//eklmn(khaos)
	char			m_arrcHead[6];
	char		   *m_pcCompleteBuffer;
	char		   *m_pcTempBuffer;
	bool			m_bPacked;
};

enum _EnumTagTypes
{
	TAGTYPE_NONE	= 0,
	TAGTYPE_STRING	= 2,
	TAGTYPE_INT		= 3,
	TAGTYPE_FLOAT	= 4,
};
typedef EnumDomain<_EnumTagTypes>	EnumTagTypes;

template <typename TagType>
class CTag
{
public:
					CTag(TagType eSpecial, uint32 iValue);
					CTag(TagType eSpecial, LPCTSTR strValue);
					CTag(LPCTSTR strName, uint32 iValue);
					CTag(LPCTSTR strName, LPCTSTR strValue);
					CTag(CFile& file);
					CTag(FILE *pFile);
	bool			WriteToFile(CFile &file); //used for CMemFiles
	bool			WriteToFile(FILE *pFile);

	TagType			GetSpecialTag() const { return m_eSpecialTag; }
	CString			GetStringValue() const { return m_strValue; }
	CString			GetTagName() const { return m_strName; }
	void			SetTagName(LPCTSTR strName) { m_strName = strName; }
	uint32			GetIntValue() const { return m_iValue; }
	void			SetIntValue(uint32 iValue) { m_iValue = iValue; }
	EnumTagTypes	GetType() const { return m_eTagType; }
	void			Clear()
	{
		m_eTagType = TAGTYPE_NONE;
		m_strName.Empty();
		m_strValue.Empty();
		m_iValue = 0;
		m_fltValue = 0.0f;
//		m_eSpecialTag = TAGTYPE_NONE;
	}

protected:
	EnumTagTypes	m_eTagType;
	CString			m_strName;
	CString			m_strValue;
	uint32			m_iValue;
	float			m_fltValue;
	TagType			m_eSpecialTag;
};

typedef CTag<EnumClientTags>	CClientTag;

typedef CTag<EnumFriendTags>	CFriendTag;

typedef CTag<EnumEmuleTags>		CEmuleTag;

typedef CTag<EnumFileTags>		CFileTag;

//	MOREVIT - This little kludge is necessary because of compiler error C3206:
//		VS2003 does not allow a template class to be used directly as a template parameter for another template
typedef CTag<EnumServerTags>	_CServerTag;

class CServerTag : public _CServerTag
{
public:
					CServerTag(EnumServerTags eSpecial, uint32 iValue)
						: _CServerTag(eSpecial,iValue) {};
					CServerTag(EnumServerTags eSpecial, LPCTSTR strValue)
						: _CServerTag(eSpecial,strValue) {};
					CServerTag(LPCTSTR strName, uint32 iValue)
						: _CServerTag(strName,iValue) {};
					CServerTag(LPCTSTR strName, LPCTSTR strValue)
						: _CServerTag(strName,strValue) {};
					CServerTag(CFile& file)
						: _CServerTag(file) {};
					CServerTag(FILE *pFile)
						: _CServerTag(pFile) {};
};